@@ -1,10 +1,10 @@
 package oauth
 
 import (
-	"fmt"
 	"net/url"
+	"strings"
 
-	"github.com/danielgtaylor/restish/cli"
+	"github.com/rest-sh/restish/cli"
 	"golang.org/x/oauth2"
 )
 
@@ -17,6 +17,9 @@ type RefreshTokenSource struct {
 	// TokenURL is used to fetch new tokens
 	TokenURL string
 
+	// Scopes to request when refreshing the token
+	Scopes []string
+
 	// EndpointParams are extra URL query parameters to include in the request
 	EndpointParams *url.Values
 
@@ -35,14 +38,21 @@ type RefreshTokenSource struct {
 func (ts *RefreshTokenSource) Token() (*oauth2.Token, error) {
 	if ts.RefreshToken != "" {
 		cli.LogDebug("Trying refresh token to get a new access token")
-		payload := fmt.Sprintf("grant_type=refresh_token&client_id=%s&refresh_token=%s", ts.ClientID, ts.RefreshToken)
-
-		params := ts.EndpointParams.Encode()
-		if len(params) > 0 {
-			payload += "&" + params
+		refreshParams := url.Values{
+			"grant_type":    []string{"refresh_token"},
+			"client_id":     []string{ts.ClientID},
+			"refresh_token": []string{ts.RefreshToken},
+			"scope":         []string{strings.Join(ts.Scopes, " ")},
 		}
 
-		token, err := requestToken(ts.TokenURL, payload)
+		// Copy any endpoint-specific parameters.
+		if ts.EndpointParams != nil {
+			for k, v := range *ts.EndpointParams {
+				refreshParams[k] = v
+			}
+		}
+
+		token, err := requestToken(ts.TokenURL, refreshParams.Encode())
 		if err == nil {
 			return token, err
 		}
